/*
 * UDSTask.c
 *
 *  Created on: 2022Äê3ÔÂ14ÈÕ
 *      Author: Zhengchao
 */
#include "UDSTask.h"

typedef struct
{
    uint8 infoDataLen;                  /* Exchange information length must be N * 4 */
    uint8 requestEnterBootloader;       /* Request enter bootloader mode flag */
    uint8 downloadAPPSuccessful;        /* Download APP successful flag */
    uint32 infoStartAddr;               /* Exchange information start address */
    uint32 requestEnterBootloaderAddr;  /* Request enter bootloader flag address */
    uint32 downloadAppSuccessfulAddr;   /* Download APP successful flag address */
} tBootInfo;

static const tBootInfo gs_stBootInfo =
{
    16u,
    0x5Au,
    0xA5u,
    INFO_START_ADDR,
    REQUEST_ENTER_BOOTLOADER_ADDR,
    DOWNLOAD_APP_SUCCESSFUL_ADDR,
};

static const uint16 g_dnpCrcTable[256u] =
{
    0x0000, 0x365E, 0x6CBC, 0x5AE2, 0xD978, 0xEF26, 0xB5C4, 0x839A,
    0xFF89, 0xC9D7, 0x9335, 0xA56B, 0x26F1, 0x10AF, 0x4A4D, 0x7C13,
    0xB26B, 0x8435, 0xDED7, 0xE889, 0x6B13, 0x5D4D, 0x07AF, 0x31F1,
    0x4DE2, 0x7BBC, 0x215E, 0x1700, 0x949A, 0xA2C4, 0xF826, 0xCE78,
    0x29AF, 0x1FF1, 0x4513, 0x734D, 0xF0D7, 0xC689, 0x9C6B, 0xAA35,
    0xD626, 0xE078, 0xBA9A, 0x8CC4, 0x0F5E, 0x3900, 0x63E2, 0x55BC,
    0x9BC4, 0xAD9A, 0xF778, 0xC126, 0x42BC, 0x74E2, 0x2E00, 0x185E,
    0x644D, 0x5213, 0x08F1, 0x3EAF, 0xBD35, 0x8B6B, 0xD189, 0xE7D7,
    0x535E, 0x6500, 0x3FE2, 0x09BC, 0x8A26, 0xBC78, 0xE69A, 0xD0C4,
    0xACD7, 0x9A89, 0xC06B, 0xF635, 0x75AF, 0x43F1, 0x1913, 0x2F4D,
    0xE135, 0xD76B, 0x8D89, 0xBBD7, 0x384D, 0x0E13, 0x54F1, 0x62AF,
    0x1EBC, 0x28E2, 0x7200, 0x445E, 0xC7C4, 0xF19A, 0xAB78, 0x9D26,
    0x7AF1, 0x4CAF, 0x164D, 0x2013, 0xA389, 0x95D7, 0xCF35, 0xF96B,
    0x8578, 0xB326, 0xE9C4, 0xDF9A, 0x5C00, 0x6A5E, 0x30BC, 0x06E2,
    0xC89A, 0xFEC4, 0xA426, 0x9278, 0x11E2, 0x27BC, 0x7D5E, 0x4B00,
    0x3713, 0x014D, 0x5BAF, 0x6DF1, 0xEE6B, 0xD835, 0x82D7, 0xB489,
    0xA6BC, 0x90E2, 0xCA00, 0xFC5E, 0x7FC4, 0x499A, 0x1378, 0x2526,
    0x5935, 0x6F6B, 0x3589, 0x03D7, 0x804D, 0xB613, 0xECF1, 0xDAAF,
    0x14D7, 0x2289, 0x786B, 0x4E35, 0xCDAF, 0xFBF1, 0xA113, 0x974D,
    0xEB5E, 0xDD00, 0x87E2, 0xB1BC, 0x3226, 0x0478, 0x5E9A, 0x68C4,
    0x8F13, 0xB94D, 0xE3AF, 0xD5F1, 0x566B, 0x6035, 0x3AD7, 0x0C89,
    0x709A, 0x46C4, 0x1C26, 0x2A78, 0xA9E2, 0x9FBC, 0xC55E, 0xF300,
    0x3D78, 0x0B26, 0x51C4, 0x679A, 0xE400, 0xD25E, 0x88BC, 0xBEE2,
    0xC2F1, 0xF4AF, 0xAE4D, 0x9813, 0x1B89, 0x2DD7, 0x7735, 0x416B,
    0xF5E2, 0xC3BC, 0x995E, 0xAF00, 0x2C9A, 0x1AC4, 0x4026, 0x7678,
    0x0A6B, 0x3C35, 0x66D7, 0x5089, 0xD313, 0xE54D, 0xBFAF, 0x89F1,
    0x4789, 0x71D7, 0x2B35, 0x1D6B, 0x9EF1, 0xA8AF, 0xF24D, 0xC413,
    0xB800, 0x8E5E, 0xD4BC, 0xE2E2, 0x6178, 0x5726, 0x0DC4, 0x3B9A,
    0xDC4D, 0xEA13, 0xB0F1, 0x86AF, 0x0535, 0x336B, 0x6989, 0x5FD7,
    0x23C4, 0x159A, 0x4F78, 0x7926, 0xFABC, 0xCCE2, 0x9600, 0xA05E,
    0x6E26, 0x5878, 0x029A, 0x34C4, 0xB75E, 0x8100, 0xDBE2, 0xEDBC,
    0x91AF, 0xA7F1, 0xFD13, 0xCB4D, 0x48D7, 0x7E89, 0x246B, 0x1235,
};
/* Get information storage CRC */
#define GetInfoStorageCRC() (*(uint16 *)(gs_stBootInfo.infoStartAddr + 14))


/* Set information CRC */
#define SetInforCRC(xCrc) ((*(uint16 *)(gs_stBootInfo.infoStartAddr + 14)) = (uint16)(xCrc))

void CreatSoftwareCrc16(const uint8 *i_pDataBuf, const uint32 i_dataLen, uint32 *m_pCurCrc)
{
    uint16 crc = 0u;
    uint32 index = 0u;
#if (defined FALSH_ADDRESS_CONTINUE) && (FALSH_ADDRESS_CONTINUE == TRUE)
    crc = *m_pCurCrc;
#endif

    for (index = 0u; index < i_dataLen; index++)
    {
        crc = ( crc >> 8 ) ^ g_dnpCrcTable[( crc ^ i_pDataBuf[index]) & 0x00ff] ;
    }

    *m_pCurCrc = (uint32)((~crc) & 0xFFFFu);
}

void CRC_HAL_CreatSoftwareCrc(const uint8_t *i_pucDataBuf, const uint32_t i_ulDataLen, uint32_t *m_pCurCrc)
{
#ifdef EN_CRC_SOFTWARE
    CreatSoftwareCrc16(i_pucDataBuf, i_ulDataLen, m_pCurCrc);
#endif
}
/* Calculate information CRC */
static uint16 Boot_CalculateInfoCRC(void)
{
    uint32 infoCrc = 0u;
    CRC_HAL_CreatSoftwareCrc((const uint8 *)gs_stBootInfo.infoStartAddr, gs_stBootInfo.infoDataLen - 2u, &infoCrc);
    return (uint16)infoCrc;
}

/* Is information valid? */
static boolean Boot_IsInfoValid(void)
{
    uint16 infoCrc = 0u;
    uint16 storageCrc = 0u;
    boolean result = FALSE;
    infoCrc = Boot_CalculateInfoCRC();
    storageCrc = GetInfoStorageCRC();

    if (storageCrc == infoCrc)
    {
        result = TRUE;
    }

    return result;
}


void RequestEnterBootloader(void)
{
    uint16 infoCrc = 0u;
    *((uint8 *)gs_stBootInfo.requestEnterBootloaderAddr) = gs_stBootInfo.requestEnterBootloader;
    infoCrc = Boot_CalculateInfoCRC();
    SetInforCRC(infoCrc);
}

/* Is download APP successful? */
boolean IsDownloadAPPSccessful(void)
{
    boolean result = FALSE;

    if (TRUE == Boot_IsInfoValid())
    {
        if (gs_stBootInfo.downloadAPPSuccessful == *((uint8 *)gs_stBootInfo.downloadAppSuccessfulAddr))
        {
            result = TRUE;
        }
    }

    return result;
}

/* Clear download APP successful flag */
void ClearDownloadAPPSuccessfulFlag(void)
{
    uint16 infoCrc = 0u;
    *((uint8 *)gs_stBootInfo.downloadAppSuccessfulAddr) = 0u;
    infoCrc = Boot_CalculateInfoCRC();
    SetInforCRC(infoCrc);
}

static boolean UDS_IsReceivedMsgIDValid(const uint32 i_receiveMsgID)
{
    boolean result = FALSE;

    if ((i_receiveMsgID == RX_FUN_ADDR_ID)
            || (i_receiveMsgID == RX_PHY_ADDR_ID))
    {
        result = TRUE;
    }

    return result;
}

boolean IsJumptoBootloader(uint32 CanID, uint8 Data[])
{
	boolean result = FALSE;

	if(UDS_IsReceivedMsgIDValid(CanID))
	{
		if(Data[0] == 0x02 && Data[1] == 0x10 && Data[2] == 0x02)
		{
			result = TRUE;
		}
	}

	return result;
}

void DoRoutinePositiveAnswer(void)
{
	uint8 AnsData[8]={0x02,0x50,0x02,0x00,0x00,0x00,0x00,0x00};
	Can_Msg_Type CanTxMsg;

	CanTxMsg.id = TX_RESP_ADDR_ID;
	CanTxMsg.idFrame = CAN_EXTENDED_ID_TYPE;
	CanTxMsg.sdu = AnsData;
	CanTxMsg.length = 8;
	CanIf_SendMessage(1, CanTxMsg);
}

boolean Boot_CheckDownlaodAPPStatus(void)
{
    boolean ret = FALSE;

    if (TRUE == IsDownloadAPPSccessful())
    {
        ClearDownloadAPPSuccessfulFlag();
    }

    return ret;
}
