/*==================================================================================================
*   Project              : RTD AUTOSAR 4.4
*   Platform             : CORTEXM
*   Peripheral           : Ftm Lpit Lptmr Port_Ci LpCmp
*   Dependencies         : none
*
*   Autosar Version      : 4.4.0
*   Autosar Revision     : ASR_REL_4_4_REV_0000
*   Autosar Conf.Variant :
*   SW Version           : 1.0.0
*   Build Version        : S32K1_RTD_1_0_0_HF01_D2109_ASR_REL_4_4_REV_0000_20210907
*
*   (c) Copyright 2020-2021 NXP Semiconductors
*   All Rights Reserved.
*
*   NXP Confidential. This software is owned or controlled by NXP and may only be
*   used strictly in accordance with the applicable license terms. By expressly
*   accepting such terms or by downloading, installing, activating and/or otherwise
*   using the software, you are agreeing that you have read, and that you agree to
*   comply with and are bound by, such license terms. If you do not agree to be
*   bound by the applicable license terms, then you may not retain, install,
*   activate or otherwise use the software.
==================================================================================================*/

#ifndef CMP_IP_VS_0_PBCFG_H
#define CMP_IP_VS_0_PBCFG_H

/**
 *   @file    Cmp_Ip_PBcfg.h
 *   @version 1.0.0
 *
 *   @brief   AUTOSAR Icu - contains the data exported by the Icu module
 *   @details Contains the information that will be exported by the module, as requested by Autosar.
 *
 *   @addtogroup cmp_icu_ip CMP IPL
 *   @{
 */
 
#ifdef __cplusplus
extern "C"{
#endif

/*==================================================================================================
*                                         INCLUDE FILES
*================================================================================================*/
#include "Cmp_Ip_Types.h"

/*==================================================================================================
 *                              SOURCE FILE VERSION INFORMATION
 *================================================================================================*/
#define CMP_IP_PBCFG_VS_0_VENDOR_ID                    43
#define CMP_IP_PBCFG_VS_0_AR_RELEASE_MAJOR_VERSION     4
#define CMP_IP_PBCFG_VS_0_AR_RELEASE_MINOR_VERSION     4
#define CMP_IP_PBCFG_VS_0_AR_RELEASE_REVISION_VERSION  0
#define CMP_IP_PBCFG_VS_0_SW_MAJOR_VERSION             1
#define CMP_IP_PBCFG_VS_0_SW_MINOR_VERSION             0
#define CMP_IP_PBCFG_VS_0_SW_PATCH_VERSION             0

/*==================================================================================================
 *                                      FILE VERSION CHECKS
 *================================================================================================*/
/* Check if source file and ICU header file are of the same vendor */
#if (CMP_IP_PBCFG_VS_0_VENDOR_ID != CMP_IP_TYPES_VENDOR_ID)
    #error "Cmp_Ip_PBcfg.h and Cmp_Ip_Types.h have different vendor IDs"
#endif
/* Check if source file and ICU header file are of the same AutoSar version */
#if ((CMP_IP_PBCFG_VS_0_AR_RELEASE_MAJOR_VERSION != CMP_IP_TYPES_AR_RELEASE_MAJOR_VERSION) || \
     (CMP_IP_PBCFG_VS_0_AR_RELEASE_MINOR_VERSION != CMP_IP_TYPES_AR_RELEASE_MINOR_VERSION) || \
     (CMP_IP_PBCFG_VS_0_AR_RELEASE_REVISION_VERSION != CMP_IP_TYPES_AR_RELEASE_REVISION_VERSION))
    #error "AutoSar Version Numbers of Cmp_Ip_PBcfg.h and Cmp_Ip_Types.h are different"
#endif
/* Check if source file and ICU header file are of the same Software version */
#if ((CMP_IP_PBCFG_VS_0_SW_MAJOR_VERSION != CMP_IP_TYPES_SW_MAJOR_VERSION) || \
     (CMP_IP_PBCFG_VS_0_SW_MINOR_VERSION != CMP_IP_TYPES_SW_MINOR_VERSION) || \
     (CMP_IP_PBCFG_VS_0_SW_PATCH_VERSION != CMP_IP_TYPES_SW_PATCH_VERSION))
#error "Software Version Numbers of Cmp_Ip_PBcfg.h and Cmp_Ip_Types.h are different"
#endif
/*==================================================================================================
 *                                       DEFINES AND MACROS
 *================================================================================================*/



#ifdef __cplusplus
}
#endif

/** @} */

#endif /* CMP_IP_VS_0_PBCFG_H */
